      PROGRAM geom_dist
      IMPLICIT NONE
      INTEGER*8 :: i,j,k,l,war
      INTEGER*8,PARAMETER :: nmod=42,nato=49,iesf=1 !iesf=0 cartesianas, iesf=1 esfericas
      INTEGER*8,PARAMETER :: imax=10 !Only the first atoms from the second one to the number imax in the relaxed 
!                                     and distorted geometries are printed.
!                                     In these first imax atoms, the first one is the metal and the
!                                     remaining ones are the ligand metal-coordinating atoms.
      REAL*8 :: pref,re,te,fe        
      REAL*8 :: rx1,ry1,rz1          
      REAL*8 :: rx2,ry2,rz2 
      REAL*8 :: rx3,ry3,rz3 
      REAL*8,PARAMETER :: thres=1.d-8,bohr=0.5291772086d0
      REAL*8,PARAMETER :: rres=1.4165d0,ceff=0.0223333d0 !rres=radial displacement to apply to each ligand
!                                                         metal-coordinating atom of the relaxed geoemtry and 
!                                                         each distorted geometry
!                                                         ceff=effective charge to apply to each ligand
!                                                         metal-coordinating atom of the relaxed geometry and 
!                                                         each distorted geometry
      REAL*8,PARAMETER :: the=0.0d0,phi=352.0d0 !0<the<180.0;0<phi<360. These angles are used to rotate the 
!                                                relaxed geometry and each distorted geometry if desired. 
!                                                First, a clock-wise rotation of angle "the" is performed around
!                                                the Y=(0,1,0) axis. Then, an anticlock-wise rotation of angle
!                                                "phi" is performed around the Z=(0,0,1) axis. If this rotation
!                                                is not desired, just set the=0.0d0 and phi=0.0d0.
!                                                This rotation option only works if iesf=1.
      REAL*8,PARAMETER :: pi=3.1415926535898d0 
      INTEGER*8,DIMENSION(nmod) :: ngdis
      REAL*8,DIMENSION(nmod) :: prefv
      REAL*8,DIMENSION(nato,3) :: geoeq,geodis,vecmod
      CHARACTER*2 atom(nato)
      WRITE(*,*) ' '
      WRITE(*,*) ' *** WARNING *** '
      WRITE(*,*) ' '
      WRITE(*,*) ' Check whether the minimum displacements and the numbe&
     &r of  distorted geometries of each mode are ordered from the first&
     & mode to the last mode with increasing harmonic frequency'
      WRITE(*,*) ' '
      WRITE(*,*) ' Check whether the list of the coordinates of the equi&
     &librium geometry is in Angstroms'
      WRITE(*,*) ' '
      WRITE(*,*) ' Check whether the coordinates of the magnetic center &
     &are found in the first row of the previous list'
      WRITE(*,*) ' '
      WRITE(*,*) ' Check whether the order of the atoms of the equilibri&
     &um geometry coincides with the order of the rows of all displaceme&
     &nt vectors'
      WRITE(*,*) ' '
      WRITE(*,*) ' If ALL answers are affirmative:'
      war=2
      DO WHILE ((war.ne.0).and.(war.ne.1)) 
        WRITE(*,*) ' Write 1 and press ENTER to continue, write 0 and pr&
     &ess ENTER to abort.'
        WRITE(*,*) ' '
        READ(*,*) war
      END DO
      WRITE(*,*) ' '
      IF (war.eq.0) THEN
        WRITE(*,*) ' Calculation aborted...'
        WRITE(*,*) ' '
        STOP
      ENDIF
      WRITE(*,*) ' Calculation in process...'
      WRITE(*,*) ' ' 
      OPEN(1,FILE='geom_dist_input',STATUS='unknown')
      DO 8 i=1,nmod !The minimum displacements (Angstroms) and the number of distorted geometries are read
        READ(1,503) prefv(i),ngdis(i)
 8    CONTINUE
      OPEN(2,FILE='geom_dist_output',STATUS='unknown')
      WRITE(2,*) ' '
      WRITE(2,*) ' Mode   min. dis.(A)   n. dist. geom.'
      WRITE(2,*) ' '
      DO 9 i=1,nmod !The minimum displacements (Angstroms) and the number of distorted geometries are written
        WRITE(2,504) i,prefv(i),2*ngdis(i)
 9    CONTINUE
      READ(1,*)
      DO 1 i=1,nato !The equilibrium geometry in cartesian coordinates (Angstroms) is read
        READ(1,*) atom(i),(geoeq(i,j),j=1,3)
 1    CONTINUE
      IF (iesf.eq.0) THEN !The equilibrium geometry printed in cartesian coordinates is never rotated
        WRITE(2,*) ' '
        WRITE(2,*) ' Equilibrium geometry (in Angstroms):'
        WRITE(2,*) ' '
        DO 2 i=2,imax !The equilibrium geometry in cartesian coordinates (Angstroms) is written 
          WRITE(2,501) i-1,(geoeq(i,j)-geoeq(1,j),j=1,3),ceff 
 2      CONTINUE
      ELSE !The equilibrium geometry printed in spherical coordinates can be previously rotated
        WRITE(2,*) ' '
        WRITE(2,*) ' Equilibrium geometry (in Angstroms,degrees,degrees)&
     &:'
        WRITE(2,*) ' '
        DO 11 i=2,imax !The equilibrium geometry in spherical coordinates (Angstroms,degrees,degrees) is written
          rx1=geoeq(i,1)-geoeq(1,1) 
          ry1=geoeq(i,2)-geoeq(1,2) 
          rz1=geoeq(i,3)-geoeq(1,3) 
          rx2=rx1*dcos((the*pi)/180.0d0)-rz1*dsin((the*pi)/180.0d0) 
          ry2=ry1 
          rz2=rx1*dsin((the*pi)/180.0d0)+rz1*dcos((the*pi)/180.0d0) 
          rx3=rx2*dcos((phi*pi)/180.0d0)-ry2*dsin((phi*pi)/180.0d0) 
          ry3=rx2*dsin((phi*pi)/180.0d0)+ry2*dcos((phi*pi)/180.0d0) 
          rz3=rz2 
          call coesf(rx3,ry3,rz3,re,te,fe) !rotation of the equilibrium geometry prior to writing it
          WRITE(2,501) i-1,re-rres,te,fe,ceff 
 11     CONTINUE
      ENDIF
      DO 3 i=1,nmod !loop for all modes from 1 to nmod
        WRITE(2,*) ' '
        WRITE(2,502) i
        WRITE(2,*) ' ' 
        READ(1,*)
        DO 5 k=1,nato !the displacement vectors of all modes are read
          READ(1,*) (vecmod(k,l),l=1,3)
 5      CONTINUE 
        pref=-prefv(i)*dble(ngdis(i))
        DO 4 j=1,2*ngdis(i)+1 !2*ngdis(i) distorted geometries of the mode i are generated
          IF (dabs(pref).gt.thres) THEN
            DO 6 k=1,nato  
              geodis(k,1)=geoeq(k,1)+pref*vecmod(k,1)  
              IF (dabs(geodis(k,1)).lt.thres) THEN
                geodis(k,1)=0.0d0
              ENDIF 
              geodis(k,2)=geoeq(k,2)+pref*vecmod(k,2)
              IF (dabs(geodis(k,2)).lt.thres) THEN
                geodis(k,2)=0.0d0
              ENDIF
              geodis(k,3)=geoeq(k,3)+pref*vecmod(k,3)
              IF (dabs(geodis(k,3)).lt.thres) THEN
                geodis(k,3)=0.0d0
              ENDIF
 6          CONTINUE
            WRITE(2,500) pref
            WRITE(2,*) ' '
            IF (iesf.eq.0) THEN 
              WRITE(2,*) ' '
              WRITE(2,*) ' Atom coordinates (in Angstroms):'
              WRITE(2,*) ' '
              DO 7 k=2,imax !the distorted geometries are written 
                WRITE(2,501) k-1,(geodis(k,l)-geodis(1,l),l=1,3),c&
     &eff 
 7            CONTINUE 
              WRITE(2,*) ' '
            ELSE 
              WRITE(2,*) ' '
              WRITE(2,*) ' Atom coordinates (in Angstroms,degrees,degree&
     &s):'
              WRITE(2,*) ' '
              DO 10 k=2,imax !the distorted geometries are written 
          rx1=geodis(k,1)-geodis(1,1) 
          ry1=geodis(k,2)-geodis(1,2) 
          rz1=geodis(k,3)-geodis(1,3) 
          rx2=rx1*dcos((the*pi)/180.0d0)-rz1*dsin((the*pi)/180.0d0) 
          ry2=ry1 
          rz2=rx1*dsin((the*pi)/180.0d0)+rz1*dcos((the*pi)/180.0d0) 
          rx3=rx2*dcos((phi*pi)/180.0d0)-ry2*dsin((phi*pi)/180.0d0) 
          ry3=rx2*dsin((phi*pi)/180.0d0)+ry2*dcos((phi*pi)/180.0d0) 
          rz3=rz2 
                CALL coesf(rx3,ry3,rz3,re,te,fe) !rotation of the equilibrium geometry prior to writing it
                WRITE(2,501) k-1,re-rres,te,fe,ceff 
 10           CONTINUE 
              WRITE(2,*) ' '
            ENDIF
          ENDIF
          pref=pref+prefv(i)
 4      CONTINUE
 3    CONTINUE
      CLOSE (1)
      CLOSE (2)
      WRITE(*,*) ' Calculation done...'
      WRITE(*,*) ' '
 500  FORMAT('  Distortion Coordinate (A): ',f8.4)
 501  FORMAT(i3,4f13.7) 
 502  FORMAT('  Mode: ',I4)
 503  FORMAT(f5.3,I4)
 504  FORMAT(' ',I4,'         ',f5.3,'        ',I4)
      END PROGRAM
      subroutine coesf(xc,yc,zc,re,te,fe)
      real*8 :: xc,yc,zc,re,te,fe
      real*8, parameter :: pi=3.1415926535898d0 
      real*8,parameter :: thres=1.d-8
      re=dsqrt(xc**2+yc**2+zc**2)    
      te=(dacos(zc/re))*(180.0d0/pi)
      fe=(atan2(yc,xc))*(180.0d0/pi)
      if (fe.le.(-thres)) fe=fe+360.0d0
      end subroutine
